@@ -21,7 +21,7 @@
 from . import matchers
 from . import signature
 from . import verification as verificationModule
-from .mock_registry import mock_registry
+from .utils import contains_strict
 
 from collections import deque
 import functools
@@ -53,88 +53,6 @@ class Invocation(object):
         params = ", ".join(args + kwargs)
         return "%s(%s)" % (self.method_name, params)
 
-    def ensure_signature_matches(self, method_name, args, kwargs):
-        sig = self.mock.get_signature(method_name)
-        if not sig:
-            return
-
-        signature.match_signature(sig, args, kwargs)
-
-
-class MatchingInvocation(Invocation):
-    @staticmethod
-    def compare(p1, p2):
-        if isinstance(p1, matchers.Matcher):
-            if not p1.matches(p2):
-                return False
-        elif p1 != p2:
-            return False
-        return True
-
-    def _remember_params(self, params, named_params):
-        if Ellipsis in params and (params[-1] is not Ellipsis or named_params):
-            raise TypeError('Ellipsis must be the last argument you specify.')
-
-        if matchers.args in params:
-            raise TypeError('args must be used as *args')
-
-        if matchers.kwargs in params or matchers.KWARGS_SENTINEL in params:
-            raise TypeError('kwargs must be used as **kwargs')
-
-        def wrap(p):
-            if p in (any, matchers.any_):
-                return matchers.any_()
-            return p
-
-        self.params = tuple(wrap(p) for p in params)
-        self.named_params = {k: wrap(v) for k, v in list(named_params.items())}
-
-
-    # Note: matches(a, b) does not imply matches(b, a) because
-    # the left side might contain wildcards (like Ellipsis) or matchers.
-    # In its current form the right side is a concrete call signature.
-    def matches(self, invocation):  # noqa: C901 (too complex)
-        if self.method_name != invocation.method_name:
-            return False
-
-        for x, p1 in enumerate(self.params):
-            # assume Ellipsis is the last thing a user declares
-            if p1 is Ellipsis:
-                return True
-
-            if p1 is matchers.ARGS_SENTINEL:
-                break
-
-            try:
-                p2 = invocation.params[x]
-            except IndexError:
-                return False
-
-            if not self.compare(p1, p2):
-                return False
-        else:
-            if len(self.params) != len(invocation.params):
-                return False
-
-        for key, p1 in sorted(
-                iter(self.named_params.items()),
-                key=lambda k_v: 1 if k_v[0] is matchers.KWARGS_SENTINEL else 0):
-            if key is matchers.KWARGS_SENTINEL:
-                break
-
-            try:
-                p2 = invocation.named_params[key]
-            except KeyError:
-                return False
-
-            if not self.compare(p1, p2):
-                return False
-        else:
-            if len(self.named_params) != len(invocation.named_params):
-                return False
-
-        return True
-
 
 class RememberedInvocation(Invocation):
     def __init__(self, mock, method_name):
@@ -148,6 +66,13 @@ class RememberedInvocation(Invocation):
                 "You tried to call a method '%s' the object (%s) doesn't "
                 "have." % (method_name, self.mock.mocked_obj))
 
+    def ensure_signature_matches(self, method_name, args, kwargs):
+        sig = self.mock.get_signature(method_name)
+        if not sig:
+            return
+
+        signature.match_signature(sig, args, kwargs)
+
     def __call__(self, *params, **named_params):
         if self.strict:
             self.ensure_mocked_object_has_method(self.method_name)
@@ -203,6 +128,88 @@ class RememberedProxyInvocation(Invocation):
         return method(*params, **named_params)
 
 
+class MatchingInvocation(Invocation):
+    @staticmethod
+    def compare(p1, p2):
+        if isinstance(p1, matchers.Matcher):
+            if not p1.matches(p2):
+                return False
+        elif p1 != p2:
+            return False
+        return True
+
+    def _remember_params(self, params, named_params):
+        if (
+            contains_strict(params, Ellipsis)
+            and (params[-1] is not Ellipsis or named_params)
+        ):
+            raise TypeError('Ellipsis must be the last argument you specify.')
+
+        if contains_strict(params, matchers.args):
+            raise TypeError('args must be used as *args')
+
+        if (
+            contains_strict(params, matchers.kwargs)
+            or contains_strict(params, matchers.KWARGS_SENTINEL)
+        ):
+            raise TypeError('kwargs must be used as **kwargs')
+
+        def wrap(p):
+            if p is any or p is matchers.any_:
+                return matchers.any_()
+            return p
+
+        self.params = tuple(wrap(p) for p in params)
+        self.named_params = {k: wrap(v) for k, v in list(named_params.items())}
+
+
+    # Note: matches(a, b) does not imply matches(b, a) because
+    # the left side might contain wildcards (like Ellipsis) or matchers.
+    # In its current form the right side is a concrete call signature.
+    def matches(self, invocation):  # noqa: C901 (too complex)
+        if self.method_name != invocation.method_name:
+            return False
+
+        for x, p1 in enumerate(self.params):
+            # assume Ellipsis is the last thing a user declares
+            if p1 is Ellipsis:
+                return True
+
+            if p1 is matchers.ARGS_SENTINEL:
+                break
+
+            try:
+                p2 = invocation.params[x]
+            except IndexError:
+                return False
+
+            if not self.compare(p1, p2):
+                return False
+        else:
+            if len(self.params) != len(invocation.params):
+                return False
+
+        for key, p1 in sorted(
+            iter(self.named_params.items()),
+            key=lambda k_v: 1 if k_v[0] is matchers.KWARGS_SENTINEL else 0
+        ):
+            if key is matchers.KWARGS_SENTINEL:
+                break
+
+            try:
+                p2 = invocation.named_params[key]
+            except KeyError:
+                return False
+
+            if not self.compare(p1, p2):
+                return False
+        else:
+            if len(self.named_params) != len(invocation.named_params):
+                return False
+
+        return True
+
+
 class VerifiableInvocation(MatchingInvocation):
     def __init__(self, mock, method_name, verification):
         super(VerifiableInvocation, self).__init__(mock, method_name)
@@ -274,6 +281,13 @@ class StubbedInvocation(MatchingInvocation):
                 "You tried to stub a method '%s' the object (%s) doesn't "
                 "have." % (method_name, self.mock.mocked_obj))
 
+    def ensure_signature_matches(self, method_name, args, kwargs):
+        sig = self.mock.get_signature(method_name)
+        if not sig:
+            return
+
+        signature.match_signature_allowing_placeholders(sig, args, kwargs)
+
     def __call__(self, *params, **named_params):
         if self.strict:
             self.ensure_mocked_object_has_method(self.method_name)
@@ -285,6 +299,9 @@ class StubbedInvocation(MatchingInvocation):
         self.mock.finish_stubbing(self)
         return AnswerSelector(self)
 
+    def forget_self(self):
+        self.mock.forget_stubbed_invocation(self)
+
     def add_answer(self, answer):
         self.answers.add(answer)
 
@@ -371,7 +388,7 @@ class AnswerSelector(object):
         pass
 
     def __exit__(self, *exc_info):
-        mock_registry.unstub(self.invocation.mock.mocked_obj)
+        self.invocation.forget_self()
 
 
 class CompositeAnswer(object):
